
module Main where


import IO 
import System ( getArgs, getProgName )
import System.FilePath.Posix
import Common.PrettyPrinter
import Parser.Lexglobaltype
import Parser.Parglobaltype
import Common.Absglobaltype
import Global.Linearity
import Global.OneTimeUnfolding
import Global.Projection
import Common.Typing
import Local.BinaryTyping
import Local.MultiTyping
import Global.WellAsserted
import CodeGeneration.Generator
import Unsafe.Coerce
import Parser.ErrM

type ParseFun  = [Token] -> P Program

myLLexer = myLexer

type Verbosity = Int


printTyping :: Typing -> String
printTyping [] = ""
printTyping ((ch,p,chs,lt):xs) = (prPrint ch)++"["++(prPrintList chs)++"]: "++(prPrint lt)++"@"++(prPrint p)++"\n \n"++(printTyping xs)

printTypingBin :: Typing -> String
printTypingBin [] = ""
printTypingBin ((ch,p,chs,lt):xs) = (prPrint ch)++": "++(prPrint lt)++"\n"++(printTypingBin xs)

doGlobal :: String -> Program -> IO()
doGlobal fname p = case p of
                     GlobalProg d -> printProjection d
                     SingleProg p -> do putStrLn ("\nTypes: \n"++(printTypingBin (binarySessionTyping p)))
                                        doWrite fname (generate p) 
                     Prog p d -> do  putStrLn ("\nWellAsserted? "++(show (
                                                                        map (wellAsserted . (\(Global c' d') -> d')) p
                                                                       )))
                                     putStrLn ("\nProjections:\n"++(prPrint projmap))
                                     putStrLn ("\nTypes: \n"++(printTyping (typingSystem (addProjectionMap projmap) d)))
                                     doWrite fname (generate d)
                                         where projmap = doProjection p

printProjection :: [GlobalSpecification] -> IO()
printProjection [] = putStr "\n"
printProjection ((Global ch d):xs) = do putStrLn ("\n\nWellAsserted? "++(show (wellAsserted d)))
                                        putStr ("\nProjections for "++(prPrint ch)++": \n")
                                        putStrLn (prPrintList (computeProjection d))
                                        printProjection xs
                                



doWrite filename str = do putStr $ "Writing generated code in \""++filename++"\"... "
                          writeFile filename str
                          putStrLn "Done."


runFile :: String -> ParseFun -> FilePath -> IO ()
runFile fn p f = putStrLn f >> readFile f >>= run fn p

run :: String -> ParseFun -> String -> IO ()
run fn p s = let ts = myLLexer s in case runP (pProgram ts) "" 1 of
                                      Bad s    -> do putStrLn "\nParse              Failed...\n"
                                                     putStrLn $ "error message: "++s
                                      Ok  tree -> do putStrLn "\nParse Successful."
                                                     -- putStrLn $ show tree
                                                     doGlobal fn tree




main :: IO ()
main = do args <- getArgs
          prn <- getProgName
          case args of
            [] -> hGetContents stdin >>= run "output.hs" pProgram
            fs:[] -> runFile (replaceExtension fs ".hs") pProgram fs
            otherwise -> putStrLn $ "Usage: ./"++prn++" [inputFile]"


