module Main where

import Network
import System.IO
import Control.Concurrent
import Control.Concurrent.Chan
import Control.Exception
import Prelude hiding (catch)
import System.Exit
--
-- Need to find a way to send a list of values: 
--  -might have to use "atomic" and several send/receive
--  -or just send a list [a,b,c]? or tuple ?
--
--  Need to carry variables in recursion (a simpler way?)



-- Thread termination synch

-- like it is said on
-- http://www.haskell.org/ghc/docs/latest/html/libraries/base/Control-Concurrent.html
-- it lets you block the main thread until all the children terminates
waitForChildren :: MVar [MVar ()] -> IO ()
waitForChildren children = do
  cs <- takeMVar children
  case cs of
    []   -> return ()
    m:ms -> do
            putMVar children ms
            takeMVar m
            waitForChildren children


-- creates a new thread within the thread syncrhonization mechanism
forkChild :: MVar [MVar ()] -> IO () -> IO ThreadId
forkChild children io = do
    mvar <- newEmptyMVar
    childs <- takeMVar children
    putMVar children (mvar:childs)
    forkIO (io `finally` putMVar mvar ())

-- [End] Thread termination synch

bodyA rec s t = do writeChan s (tail rec)
                   sel <- readChan t
                   if (sel == "ending")
                    then putStrLn "A done."
                    else do
                     writeChan s "Go on."
                     bodyA (tail rec) s t

bodyB s t = do tmp <- readChan s
               if (tmp == "0")
                then do
                  writeChan t "ending"
                  putStrLn "B Done."
                else do
                  writeChan t "cont"
                  ss <- readChan s
                  putStrLn $ show ss
                  bodyB s t


main = do
  children <- newMVar [] -- synch
  s <- newChan
  t <- newChan


  forkChild children $ do -- A
         putStrLn "Starting A..."
         soca <- listenOn (PortNumber 6230)
         (ha, ho, nu) <- accept soca 
         putStrLn $ "A accepted: "++(show ho)++":"++(show nu)
         hPutStr ha "Hello, I am A\n"
         hFlush ha
         putStrLn $ "A has sent data."
         sClose soca
 
  
 
  waitForChildren children -- synch
  putStrLn "End."



connectMe c m = if c <= m
                then do hc <- catch (connectTo ("localhost") (PortNumber 6230))
                           (\e ->  do putStrLn $ "Error: "++(show e)++"\n retry..."
                                      connectMe (c+1) m
                           )
                        return hc
                else exitWith (ExitFailure 1)